import re
from enum import Enum
from collections import OrderedDict
from collections import namedtuple

import util


class PraatLTASDatoteka:
    class Stanje(Enum):
        CITANJE_VRIJEDNOSTI = 1
        DRUGO = 2

    def __init__(self, ime_datoteke):
        self.__ime_datoteke = ime_datoteke
        self.__uzorak = OrderedDict()

    def ucitaj_sve(self):
        """
        :return:
        """
        ltas = []
        with open(self.__ime_datoteke) as f:
            for redak in f:
                s = redak.strip()
                if s is not None \
                        and len(s) > 0 \
                        and s[0] == 'z' \
                        and '=' in redak:
                    ltas += [float(s[s.find('=') + 2:])]  # +2 zbog razmaka

        return ltas

    def ucitaj(self):
        """
        :return: Uredjena mapa (OrderedDict) gdje je kljuc naziv glasa, a
         vrijednost lista svih LTASova za taj glas. Na primjer, ('a', [LTAS-1, 
         LTAS-2, ..., LTAS-n]). LTAS-k je lista.
        """
        stanje = PraatLTASDatoteka.Stanje.DRUGO
        re_ltas = re.compile(r"z [\[1\] \[0-9*\]\W*]")
        re_naziv = re.compile(r'name = \"(\w*)\"')
        re_vrijednost = re.compile(r'= (-?[0-9]*.[0-9]*)')
        ime = None
        ltas = []
        with open(self.__ime_datoteke) as f:
            for redak in f:
                naziv = re_naziv.search(redak)
                if naziv is not None:
                    ime = naziv.group(1)
                elif redak.strip() == "z [1]:":
                    stanje = PraatLTASDatoteka.Stanje.CITANJE_VRIJEDNOSTI
                else:
                    if stanje == PraatLTASDatoteka.Stanje.CITANJE_VRIJEDNOSTI:
                        if re_ltas.match(redak.strip()):
                            v = re_vrijednost.search(redak).group(1)
                            # dodaj vrijednost za trenutni LTAS
                            ltas += [float(v)]
                        else:  # dosli smo do zadnje vrijednosti jednog LTASa
                            """
                            * preskacemo sve do pocetka sljedeceg niza LTAS 
                              vrijednosti
                            * ucitavamo samo prvih 120 vrijednosti, t.j. do 
                              12 kHz

                            U izrazu ltas[:120] + [1] mozemo maknuti +[1], ali 
                            onda treba ponovo trenirati mrezu!
                            """
                            stanje = PraatLTASDatoteka.Stanje.DRUGO
                            if ime in self.__uzorak:
                                self.__uzorak[ime] += [ltas[:120] + [1]]
                            else:
                                self.__uzorak[ime] = [ltas[:120] + [1]]
                            ltas = []

        self.__uzorak[ime] = [ltas[:120] + [1]]  # dodaj zadnji LTAS
        return self.__uzorak

    def vrijednosti(self):
        return self.__uzorak


def vrijeme_vrijednost_dat(direktorij, ime_datoteke, interval=None):
    popis = []
    with open(f'{util.konfig("datoteka", "dir")}{direktorij}\{ime_datoteke}') \
            as f:
        for redak in f:
            if ':' in redak:
                # zadnji znak preskacemo jer sadrzi '\n'
                par = redak[:-1].split(':')
                try:
                    vrijeme = round(float(par[0]), 2)
                    vrijednost = round(float(par[1]), 2)
                    if interval is not None:
                        if interval[0] <= vrijeme <= interval[1]:
                            popis += [(vrijeme, vrijednost)]
                    else:
                        popis += [(vrijeme, vrijednost)]
                except:
                    pass  # ako Praat nije generirao vrijednost, idemo dalje

    return popis


class Titlovi:
    def __init__(self, direktorij):
        self._direktorij = direktorij
        self._titlovi = []

    def ucitaj(self):
        datoteka = f'{util.konfig("datoteka", "dir")}{self._direktorij}' \
            f'/titlovi.txt'
        titl_info = namedtuple('TitlInfo', ['titl', 'vrijeme_od', 'vrijeme_do'])
        popis_titlova = []
        for red in open(datoteka, 'r', encoding='utf-8'):
            if red.strip() == '':
                continue
            m = re.match('^[0-9]{2}:[0-9]{2}:[0-9]{2},[0-9]{3}', red)
            if m is not None:
                vrijeme = red[m.start():m.end()].split(':')
                minute = int(vrijeme[1])
                sekunde = float(vrijeme[2].replace(',', '.'))
                titl = red[red.index('=') + 1:-1]  # -1 da izbaci \n
                popis_titlova += [titl_info(
                        titl,
                        (minute, sekunde),
                        (None, None))]

                # pocetak trenutnog titla je zavrsetak prethodnog
                if len(popis_titlova) > 1:
                    popis_titlova[-2] = titl_info(
                            popis_titlova[-2].titl,
                            popis_titlova[-2].vrijeme_od,
                            (minute, sekunde)
                    )

        return popis_titlova[:-1]  # zadnji je "..." pa nam ne treba


if __name__ == '__main__':
    t = Titlovi('/detekcija/proba_11_z5')
    print(t.ucitaj())
