"""
Rastavljanje rijeci na slogove.
"""


class Slogovi:
    uzorci = {
        'podjela': {
            'VCV': 'V-CV',
            'VCCV': 'VC-CV',
            'VCCCV': 'V-CCCV',
            'VCCCCV': 'V-CCCCV',
            'VCCCCCV': 'V-CCCCCV',
        },
        'cjelina': {'ije', 'naest'},
    }

    izuzeci = {
        'dvije': 'dvi-je',
        'prije': 'pri-je',
    }

    vokali = {'a', 'e', 'i', 'o', 'u'}

    def __init__(self, rijec):
        self.rijec = rijec
        self.oblik_rijeci = ''.join(
            map(lambda c: 'V' if c in Slogovi.vokali else 'C',
                rijec))

    @staticmethod
    def konsonant(c):
        if c not in Slogovi.vokali and str.isalpha(c):
            return True
        else:
            return False

    @staticmethod
    def sve_konsonanti(rijec):
        r = filter(lambda c: str.isalpha(c) and c in Slogovi.vokali, rijec)
        if len(list(r)) == 0:
            return True
        else:
            return False

    @staticmethod
    def pravilo_za_r(rastavljena_rijec):
        """
        Pravilo za 'r' je u stvari izuzetak od pravila u gornjoj mapi, pa
        nam treba posebna funkcija za to.
        :return:
        """
        rezultat = rastavljena_rijec
        pozicija_r = rezultat.find('r')
        while pozicija_r >= 0:
            # ako je 'r' na pocetku onda rC -> r-C (ako C nije 'j')
            if pozicija_r == 0 and \
                    Slogovi.konsonant(rezultat[pozicija_r + 1]) and \
                    rezultat[pozicija_r + 1] != 'j':
                rezultat = 'r-' + rezultat[1:]
            # ako je oblik CrC i drugi C nije 'j' onda CrC -> Cr-C
            elif len(rezultat) - 1 > pozicija_r > 0 and \
                    Slogovi.konsonant(rezultat[pozicija_r - 1]) and \
                    Slogovi.konsonant(rezultat[pozicija_r + 1]) and \
                    rezultat[pozicija_r + 1] != 'j':
                rezultat = rezultat[:pozicija_r + 1] + '-' + \
                           rezultat[pozicija_r + 1:]
            # sta ako je 'r' na kraju rijeci (masakr, žanr)?
            # vjerojatno VCr->V-Cr
            elif pozicija_r == len(rezultat) - 1 and \
                    Slogovi.konsonant(rezultat[pozicija_r - 1]) and \
                    rezultat[pozicija_r - 2] in Slogovi.vokali:
                rezultat = rezultat[:pozicija_r - 1] + '-' + \
                           rezultat[pozicija_r - 1:]

            pozicija_r = rezultat.find('r', pozicija_r + 1)

        return rezultat

    @staticmethod
    def zamijeni(oblik_rijeci, pozicija, oblik):
        return oblik_rijeci[:pozicija] + oblik + \
               oblik_rijeci[pozicija + len(oblik) - 1:]

    @staticmethod
    def oblik_u_rastavljenu_rijec(oblik, rijec):
        duljine_slogova = map(lambda s: len(s), oblik.split('-'))
        slogovi = ''
        r = rijec
        for d in duljine_slogova:
            if slogovi != '':
                slogovi += '-'
            slogovi += r[:d]
            r = r[d:]

        return slogovi

    def rastavlja_cjelinu(self, pozicija_nerastavljenog_oblika):
        for cjelina in Slogovi.uzorci['cjelina']:
            rastavljena_rijec = self.oblik_u_rastavljenu_rijec(
                self.oblik_rijeci,
                self.rijec)
            pozicija_cjeline = rastavljena_rijec.find(cjelina)
            if pozicija_cjeline >= 0:  # ako ta cjelina postoji u rijeci
                cjelina_od, cjelina_do = pozicija_cjeline, \
                                         pozicija_cjeline + len(cjelina) - 1
                # Ako nerastavljeni oblik pocinje na zadnjem slovu cjeline
                # to ne rastavlja cjelinu (jer '-' dolazi nakon prvog znaka
                # nerastavljenog oblika, kako se vidi u mapi).
                if cjelina_od <= pozicija_nerastavljenog_oblika < cjelina_do:
                    return True

        return False

    def podjela(self):
        # Pretpostavljam da ako se rijec sastoji samo od konsonanata onda je
        # automatski jednoslozna.
        if Slogovi.sve_konsonanti(self.rijec):
            return [self.rijec]

        # za izuzetke unaprijed znamo kako ih rastaviti
        if self.rijec in Slogovi.izuzeci:
            return Slogovi.izuzeci[self.rijec]

        # sve ostalo ide po pravilima
        try:
            for oblik in Slogovi.uzorci['podjela']:
                pozicija_oblika = self.oblik_rijeci.find(oblik)
                while pozicija_oblika >= 0:
                    podjela = Slogovi.uzorci['podjela'][oblik]
                    if not self.rastavlja_cjelinu(pozicija_oblika):
                        self.oblik_rijeci = Slogovi.zamijeni(self.oblik_rijeci,
                                                             pozicija_oblika,
                                                             podjela)

                    pozicija_oblika = self.oblik_rijeci.find(oblik,
                                                             pozicija_oblika + 1)

            rastavljena_rijec = Slogovi.oblik_u_rastavljenu_rijec(
                self.oblik_rijeci,
                self.rijec)

            return Slogovi.pravilo_za_r(rastavljena_rijec).split('-')
        except Exception:
            return []


if __name__ == '__main__':
    _rijeci = ['cvijet', 'smiješak', 'smiješiti', 'dvanaestica', 'nadogradnja',
               'vrijeme', 'problematika', 'jedanaest', 'jedanaestica',
               'prst', 'istra', 'istražiti', 'prelijepo', 'smjeti', 'vrtovi',
               'dvije', 'prije', 'rješenje', 'rzati', 'vrjednovati', 'pijem',
               'umro', 'istro', 'zahrđati', 'pohrvati', 'zahrzati', 'masakr',
               'žanr', 'uzorci', 'snijeg', 'promijeniti', 'kvrga', 'rzao',
               'odredio', 'prijašnji', 'akter', 'krpa', 'podrže']

    for _rijec in _rijeci:
        sl = Slogovi(_rijec)
        print(sl.podjela())
