import util


class Cvor:
    memo_id = 0  # ovo je samo info za debugiranje

    def __init__(self, dubina, vrijednost, indeks):
        self.prethodnik = None
        self.sljedbenici = []
        self.vrijednost = vrijednost
        self.ukupno = vrijednost
        self.putanja = []
        self.indeks = indeks
        self.dubina = dubina
        self.id = Cvor.memo_id
        Cvor.memo_id += 1

    def __repr__(self):  # info za debugiranje
        return '<Cvor>'

    def dodaj(self, cvor):
        cvor.prethodnik = self
        self.sljedbenici += [cvor]
        return cvor

    def sljedbenik(self, cvor):
        if len(self.sljedbenici) == 0:
            return self.dodaj(cvor)
        else:
            cvor.prethodnik = self
            self.sljedbenici[-1] = cvor
            return cvor

    @staticmethod
    def ispisi(cvor, oznaka, dubina=0):
        print(str(dubina) + oznaka, ' ' * (dubina * 2), cvor.indeks,
              '(' + str(cvor.vrijednost) + '/' + str(cvor.ukupno) + ')',
              cvor.putanja, '#' + str(cvor.id) if cvor.id is not None else '')
        for e in cvor.sljedbenici:
            Cvor.ispisi(e, oznaka, dubina + 1)


def nadji_poravnanje(c_glasovi, c_tekst, c_maks_razmak, fn_procjena,
                     ispis=True):
    """
    Ovaj je algoritam detaljno opisan u disertaciji.

    :param c_glasovi:
    :param c_tekst:
    :param c_maks_razmak:
    :param fn_procjena:
    :param ispis:

    :return: Lista parova (indeks grupe glasova, grupa glasova), kao
    [(0, ['z', 's', 'z']), (2, ['a', 'o']), (3, ['r', 'r', 'r']),
    (4, ['e', 'a', 'a', 'a'])]
    U ovom primjeru tekst je 'soba'. Indeks prvog para znaci da se grupa
    ['z', 's', 'z'] povezuje sa slovom 's', grupa ['a', 'o'] sa slovom 'o',
    grupa ['r', 'r', 'r'] sa slovom 'b' i grupa ['e', 'a', 'a', 'a'] sa slovom
    'a'. Indeks na kojem se nalazi cijeli par, kao sto je (2, ['a', 'o']) koji
    se nalazi na indeksu 1 (ne 2), oznacava indeks slova s kojim se doticna 
    grupa poravnava. U ovom primjeru, grupa ['a', 'o'] poravnava se sa slovom 
    na indeksu 1 ('o') jer se taj par nalazi na indeksu 1.
    """
    memo = {}

    def _izdvoji_kombinaciju(cvor, popis):
        i = cvor.indeks
        popis += [(i, c_glasovi[i])]
        if len(cvor.putanja) > 1:
            _izdvoji_kombinaciju(cvor.putanja[1], popis)

        return popis

    najbolji_cvor = None

    def _trazi(
            dubina,
            indeks,
            indeks_roditelja,
            put,
            ukupno,
            cvor,
            c_maks_dubina,
            c_maks_indeks):

        nonlocal najbolji_cvor

        # memoizacija
        if 0 < dubina < c_maks_dubina:
            kljuc = str(dubina - 1) + ':' + str(indeks - 1)
            if kljuc in memo:
                memo_cvor = memo[kljuc]
                if ispis:
                    Cvor.ispisi(memo_cvor, 'M', dubina)

                # Trenutni cvor nam ne treba pa memoizirano stablo dodajemo
                # kao sljedbenik prethodnika.
                cvor.prethodnik.sljedbenik(memo_cvor)

                # Kljucna naredba - ovdje prekidamo nepotrebno ponavljanje
                # kombinacija.
                return memo_cvor
            else:
                # Dodaj novi memo (samo inicijalni podkorijen - sljedbenici ce
                # se nadovezati kasnije).
                memo[kljuc] = cvor

        # _ispisi_cvor(cvor, dubina)
        if ispis:
            Cvor.ispisi(cvor, ':', dubina)

        if dubina >= c_maks_dubina:
            cvor.putanja += [cvor.indeks]
            return cvor

        for i in range(indeks, c_maks_indeks + 1):
            # dubina pocinje od -1
            # preskacemo razmake vece od maks_razmak
            if dubina >= 0 and i > indeks_roditelja + c_maks_razmak:
                return cvor

            # ovaj cvor ide na sljedecu razinu
            procjena = fn_procjena(c_tekst, c_glasovi, indeks, dubina + 1)
            sljedbenik = cvor.dodaj(Cvor(dubina + 1, procjena, i))
            zadnji_cvor = _trazi(dubina + 1, indeks + 1, i, put + [i],
                                 ukupno + sljedbenik.vrijednost,
                                 sljedbenik, c_maks_dubina, c_maks_indeks)

            # Nastavljamo od indeksa koji je za 1 veci od trenutnog indeksa na
            # ovoj dubini.
            indeks += 1

            if cvor.vrijednost + zadnji_cvor.ukupno > cvor.ukupno:
                cvor.ukupno = cvor.vrijednost + zadnji_cvor.ukupno
                cvor.putanja = [zadnji_cvor.id, zadnji_cvor]

            if dubina == 0:
                if najbolji_cvor is None:
                    najbolji_cvor = (cvor.id, round(cvor.ukupno, 2), cvor)
                elif najbolji_cvor[1] < cvor.ukupno:
                    najbolji_cvor = (cvor.id, round(cvor.ukupno, 2), cvor)

        return cvor

    korijen = Cvor(-1, 0, -1)

    # korijen je na dubini -1
    if len(c_glasovi) < len(c_tekst):
        _trazi(korijen.dubina, 0, 0, [], 0, korijen, len(c_glasovi) - 1,
               len(c_tekst) - 1)
    else:
        _trazi(korijen.dubina, 0, 0, [], 0, korijen, len(c_tekst) - 1,
               len(c_glasovi) - 1)

    if ispis:
        print('\n*** NAJBOLJI CVOR')
        Cvor.ispisi(najbolji_cvor[2], '>')

    # print('najbolji cvor:', najbolji_cvor)
    kombinacija = _izdvoji_kombinaciju(najbolji_cvor[2], [])
    return kombinacija, najbolji_cvor[1]  # kombinacija i bodovi


def poravnaj_slova_s_glasovima(uredjen_tekst, tabela_glasova):
    def _procjena(tekst, glasovi, indeks, dubina):
        try:
            dodatni_bodovi = {
                'ieaou': 0.3,
                'čš': 3.5,
                'cs': 3.5,
                'đžz': 3.5,
            }

            slovo = tekst[dubina]

            # Ovo baca iznimku ako je indeks veci od broja glasova, sto se
            # desava kada glas ili grupa glasova nije mogla biti poravnata sa
            # slovom.
            gl = glasovi[indeks]

            r = 0
            if slovo in gl:
                r = 0.5  # glasovima ide 0.5 boda jer se ne zna jesu li ispravni

            # ako je glas jedan od onih koje mreza bolje prepoznaje dodaj bodove
            for grupa_glasova in dodatni_bodovi.keys():
                # jesu li i slovo i glas u istom skupu?
                if slovo in grupa_glasova and \
                        (len(set(gl) & set(grupa_glasova)) > 0):
                    r += dodatni_bodovi[grupa_glasova]
                    break

            return r
        except:
            return 0  # glas nije povezan s tekstom

    # Izdvoji samo glasove (t.j. skupove glasova za svaki element u izlaznom
    # popisu).
    grupe_glasova = list(map(lambda redak: set(redak['grupa_glasova']),
                             tabela_glasova))
    vremena = list(map(lambda redak: (redak['od'], redak['do']),
                       tabela_glasova))

    pozicije, bodovi = nadji_poravnanje(
            grupe_glasova,
            uredjen_tekst,
            2,
            _procjena,
            ispis=False)

    # ovdje se radi konacni popis poravnatih slova s glasovima u formatu
    # [[slovo, (od, do)], ...]
    popis = []
    for i in range(len(uredjen_tekst)):
        # ako su iscrpljene sve grupe glasova, prekidamo
        if i >= len(pozicije):
            break

        popis += [(uredjen_tekst[i], vremena[pozicije[i][0]])]

    return popis, bodovi


def napravi_tabelu_glasova(
        glasovi,
        pocetak_zvuka,
        zavrsetak_zvuka):
    """
    Daje popis glasova organiziranih u tabelu <od, do, grupa_glasova>.

    :param glasovi: Direktorij s podacima generiranih od strane Praata.
    :param pocetak_zvuka: Pocetak segmenta zvuka.
    :param zavrsetak_zvuka: Zavrsetak segmenta zvuka.
    :return: Lista mapa s informacijama o grupi glasova.
    """
    tabela = []
    gr_gl = []  # izdvojena statistika iz grupe glasova
    vremena = []
    for e in glasovi:
        od = e[0]
        if pocetak_zvuka is not None \
                and od != 0 \
                and od < pocetak_zvuka:
            continue

        do = e[1]
        grupa_glasova = e[2]  # ovo je u tekstualnom obliku (niz slova)
        gr_gl += [e[3]]
        vremena += [(od, do)]

        # svaki redak je mapa s poljima koji odgovaraju stupcima tablice
        tabela += [{
            'od': od,
            'do': do,
            'grupa_glasova': grupa_glasova,
            # 'statistika': e[3],
        }]

        if (zavrsetak_zvuka is not None
                and do != 0
                and do > zavrsetak_zvuka):
            break

    return tabela


def intervali_rijeci(priprema_teksta_obj, poravnanje):
    """
    Razdvaja popis glasova prema granicama rijeci (oznacenim s ' ').

    :return: 
    """
    # niz prvo mora imati sva slova iz titla (bez konverzije)
    izvorni_niz = priprema_teksta_obj.izvorni_niz(poravnanje)
    intervali_rijeci = []
    jedan_interval = []
    for e in izvorni_niz:
        if e == ' ':  # zavrsava rijec?
            intervali_rijeci += [jedan_interval]
            jedan_interval = []
        else:
            jedan_interval += [e]

    '''
    Ovo daje niz s informacijama o poziciji glasova; svaka podlista je jedna 
    rijec. Primjer rijeci 'su':
        [..., [('s', (21.9523, 22.06)), ('u', (22.07, 22.086))], ...]
    '''
    intervali_rijeci += [jedan_interval]  # dodaj ostatak
    rezulat = {}
    for elementi_rijeci in intervali_rijeci:
        try:
            interval = (round(elementi_rijeci[0][1][0], 2),
                        round(elementi_rijeci[-1][1][0], 2))
            rijec = ''
            for e in elementi_rijeci:
                rijec += e[0]
            obradjena_rijec = util.PripremaTeksta(rijec).obradjen_tekst()
            rezulat[obradjena_rijec] = interval
        except:
            # print('*** GRESKA:', elementi_rijeci)
            pass

    return rezulat


# ==============================================================================


if __name__ == '__main__':
    def g_procjena(c_tekst, c_glasovi, indeks, dubina):

        def _dodatni_bodovi(s, g):
            for k in dodatni_bodovi.keys():
                # jesu li i slovo i glas u istom skupu?
                if s in k and (len(set(g) & set(k)) > 0):
                    return dodatni_bodovi[k]

            return 0

        dodatni_bodovi = {
            'ieaou': 0.3,
            'čš': 3.5,
            'cs': 3.5,
            'đžz': 3.5,
        }

        slovo = c_tekst[dubina]
        glasovi = c_glasovi[indeks]

        r = 0
        if slovo in glasovi:
            r = 0.5  # glasovima ide 0.5 boda jer se ne zna jesu li ispravni

        # Ako je glas jedan od onih koje mreza bolje prepoznaje dodaju se
        # dodatni bodovi.
        r += _dodatni_bodovi(slovo, glasovi)
        return r


    def test_podudaranje():
        grupe = [
            ['z', 's', 'z'],
            ['m', 'n'],
            ['a', 'o'],
            ['r', 'r', 'r'],
            ['e', 'a', 'a', 'a'],
            ['r', 'v', 'r', 'v'],
            ['e', 'a', 'e'],
        ]

        tekst = 'soba'

        k, b = nadji_poravnanje(grupe, tekst, 2, g_procjena, True)
        print(k, '\nBODOVI:', b)


    test_podudaranje()
