"""
* Modul koji sluzi kao pocetak programa.
"""

import neuralna_mreza as nm
import util
import detektor_rucni
import poravnavanje


def poravnanje_titla(podaci):
    """
    Poravnanje titla sa zvukom (vidi disertaciju).

    :param podaci:
    :return: None
    """
    direktorij = podaci['direktorij']
    print('\n\n##########################################', direktorij)

    popis_titlova = util.ucitaj_titlove(direktorij)
    print(popis_titlova)
    priprema_teksta_obj = util.PripremaTeksta(popis_titlova[0][2])
    uredjen_tekst = priprema_teksta_obj.obradjen_tekst()
    glasovi = nm.klasificiraj_glasove(direktorij)
    najbolji = 0, 0, 0, None  # (bodovi, tabela)

    # pomaci05 = [(0, 0), (-0.5, -0.5), (-0.5, 0.5), (0.5, -0.5), (0.5, 0.5)]
    pomaci1 = [
        # (0, 0),
        (0, -1),
        (0, 1),
        (-1, 0),
        (-1, -1),
        (-1, 1),
        (1, 0),
        (1, -1),
        (1, 1),
    ]

    for titl_info in popis_titlova:
        od = titl_info[0]
        do = titl_info[1]

        # kada poravnanje vise nema dovoljno glasova doci ce do iznimke
        for pomak in pomaci1:
            try:
                tabela_glasova = poravnavanje.napravi_tabelu_glasova(
                        glasovi,
                        od + pomak[0],
                        do + pomak[1])
                poravnanje, bodovi = poravnavanje.poravnaj_slova_s_glasovima(
                        uredjen_tekst,
                        tabela_glasova)
                print(pomak, (od + pomak[0], do + pomak[1]), '=', bodovi)
                if bodovi > najbolji[0]:
                    najbolji = bodovi, (od + pomak[0], do + pomak[1])
            except Exception as e:
                print(pomak, (od + pomak[0], do + pomak[1]), '?')
                pass

        print('\nNAJBOLJI:', najbolji[1], '=', najbolji[0])
        prikazi_rezultate(
                podaci,
                najbolji[1],
                prikazi_tabelu_glasova=False,
                prikazi_prozodiju=False)


# =============================================================================


def analiza(
        izvorni_tekst,
        intervali_rijeci,
        interval_zvuka,
        direktorij,
        prikazi_tabelu_glasova=True,
        prikazi_prozodiju=True,
        margina=0.0):
    """

    :param prikazi_prozodiju:
    :param prikazi_tabelu_glasova:
    :param interval_zvuka: Segment zvuka za analizu (od, do)
    :param izvorni_tekst: Tekst onoga sto je izreceno (bez bilo kakvog
    formatiranja). FORMAT: string
    :param intervali_rijeci: Popis rijeci izrecenih na zadanom segmentu snimke
    i njhova pocetna i zavrsna vremena (odredjena slusanjem).
    FORMAT: [(rijec, od, do), ...]
    :param direktorij: Direktorij u kojem se nalaze podaci za prepoznavanje.
    :param margina: Koliko se vremena (u sekundama) uzima prije i poslije
    intervala zvuka. FORMAT: realni broj

    :return: None
    """
    print('\n\n>>>', direktorij)

    priprema_teksta_obj = util.PripremaTeksta(izvorni_tekst)
    uredjen_tekst = priprema_teksta_obj.obradjen_tekst()
    # od, do = intervali_rijeci[0][1], intervali_rijeci[-1][2]
    od, do = interval_zvuka

    # *** osnovni koraci procesa ***

    # --- 1: prepoznavanje glasova od strane neuralne mreze

    glasovi = nm.klasificiraj_glasove(direktorij)
    intenzitet = util.ucitaj_prozodijske_podatke(
            direktorij,
            'int_10ms.txt',
            'int_puls.txt')
    ton = util.ucitaj_prozodijske_podatke(
            direktorij,
            'pitch_10ms.txt',
            'pitch_puls.txt')

    # Treba obuhvatiti malo veci dio zvuka (glasova) tako da je glasova vise
    # od slova. Ovaj popis glasova se koristi za poravnanje.
    tabela_glasova = poravnavanje.napravi_tabelu_glasova(
            glasovi,
            od - margina,
            do + margina)

    if prikazi_tabelu_glasova:
        print('\n\n*** KLASIFIKACIJA GLASOVA (GRUPIRANO) ***')
        util.prikazi_tabelu_glasova(tabela_glasova)

    # --- 2
    poravnanje, bodovi = poravnavanje.poravnaj_slova_s_glasovima(
            uredjen_tekst,
            tabela_glasova)
    print(f'(bodovi: {bodovi})\n')

    # --- 3
    tabela = detektor_rucni.izdvoji_rijeci(
            intervali_rijeci,
            tabela_glasova,
            poravnanje,
            intenzitet,
            ton,
            priprema_teksta_obj)

    detektor_rucni.analiziraj_prozodiju(
            tabela,
            tabela_glasova,
            odstupanje_ton=0.2,
            odstupanje_int=0.05,
            odstupanje_trajanje=0.7,
            velicina_prozora=6)

    print('\n\n*** PORAVNANJE ***')
    util.prikazi_tabelu_rijeci(tabela, ime_datoteke=direktorij + '.csv')

    if prikazi_prozodiju:
        grupe = detektor_rucni.analiza_tempa(tabela, min_br_slogova=5)

        print('\n\n*** PROCJENA VARIJACIJE TEMPA ***')
        util.prikazi_tempo(grupe, prikazi_graf=False)


# =============================================================================
# =============================================================================
# =============================================================================


primjer_1 = {
    'direktorij': 'primjer_1',
    'izvorni_tekst': 'a meni ostaje da pozovem još jednom sve da ovu akciju'
                     ' podrže i pozivima na broj ali i uplatama na taj račun',
    'intervali_rijeci': [
        ('a', 10.38, 10.47),
        ('meni', 10.48, 10.74),
        ('ostaje', 10.75, 11.12),
        ('da', 11.12, 11.27),
        ('pozovem', 11.39, 11.75),
        ('još', 11.75, 11.95),
        ('jednom', 11.96, 12.2),
        ('sve', 12.21, 12.45),
        ('da', 12.48, 12.57),
        ('ovu', 12.57, 12.71),
        ('akciju', 12.72, 13),
        ('podrže', 13.1, 13.55),
        ('i', 13.9, 14),
        ('pozivima', 14, 14.6),
        ('na', 14.59, 14.72),
        ('broj', 14.72, 14.95),
        ('ali', 14.96, 15.25),
        ('i', 15.26, 15.33),
        ('uplatama', 15.35, 15.97),
        ('na', 15.97, 16.08),
        ('taj', 16.11, 16.28),
        ('račun', 16.28, 16.57),
    ]
}

primjer_2 = {
    'direktorij': 'primjer_2',
    'izvorni_tekst': 'istodobno su nakon oproštaja od saborske većine u'
                     ' zagrebu članovi mosta u metkoviću dočekani kao'
                     ' sportske zvijezde',
    'intervali_rijeci': [
        ('istodobno', 21.35, 21.72),
        ('su', 21.73, 21.92),
        ('nakon', 21.9, 22.25),
        ('oproštaja', 22.3, 22.8),
        ('od', 22.8, 22.85),
        ('saborske', 22.85, 23.39),
        ('većine', 23.4013, 23.8),
        ('u', 23.8, 23.875),
        ('zagrebu', 23.9, 24.27),
        ('članovi', 24.25, 24.64),
        ('mosta', 24.7, 25),
        ('u', 25.05, 25.11),
        ('metkoviču', 25.11, 25.6),
        ('dočekani', 25.65, 26.08),
        ('kao', 26.15, 26.2854),
        ('sportske', 26.29, 26.775),
        ('zvijezde', 26.8, 27.38),
    ]
}

primjer_3 = {
    'direktorij': 'primjer_3',
    'izvorni_tekst': 'u ime tih važnijih stvari odgovorit ću im javnom'
                     ' šutnjom čak i na ono na što u nekim drugim danima',
    'intervali_rijeci': [
        ('u', 7.29, 7.33),
        ('ime', 7.33, 7.5),
        ('tih', 7.55, 7.72),
        ('važnijih', 7.72, 8.1),
        ('stvari', 8.13, 8.55),
        ('odgovorit', 8.8, 9.2),
        ('ću', 9.2, 9.31),
        ('im', 9.31, 9.47),
        ('javnom', 9.47, 9.85),
        ('šutnjom', 9.85, 10.33),
        ('čak', 10.56, 10.74),
        ('i', 10.74, 10.78),
        ('na', 10.78, 10.9),
        ('ono', 10.9, 11.15),
        ('na', 11.27, 11.4),
        ('što', 11.4, 11.62),
        ('u', 11.62, 11.66),
        ('nekim', 11.66, 11.96),
        ('drugim', 11.96, 12.27),
        ('danima', 12.27, 12.71),
    ]
}

primjer_4 = {
    'direktorij': 'primjer_4',
    'izvorni_tekst': 'odbor za ustav poslovnik i politički sustav treba reći'
                     ' tko ga može zamijeniti odnosno dati mišljenje o tome',
    'intervali_rijeci': [
        ('odbor', 7.85, 8.11),
        ('za', 8.11, 8.26),
        ('ustav', 8.26, 8.58),
        ('poslovnik', 8.66, 9.11),
        ('i', 9.19, 9.24),
        ('politički', 9.32, 9.78),
        ('sustav', 9.78, 10.23),
        ('treba', 10.33, 10.56),
        ('reći', 10.56, 10.83),
        ('tko', 10.93, 11.05),
        ('ga', 11.05, 11.15),
        ('može', 11.16, 11.42),
        ('zamijeniti', 11.42, 12.05),
        ('odnosno', 12.12, 12.72),
        ('dati', 12.72, 12.99),
        ('mišljenje', 12.99, 13.48),
        ('o', 13.48, 13.55),
        ('tome', 13.65, 13.89),
    ]
}

primjer_5 = {
    'direktorij': 'primjer_5',
    'izvorni_tekst': 'i u utorak će mjestimice biti kiše češće u gorju a s'
                     ' obzirom na to da će do nas stići hladniji zrak',
    'intervali_rijeci': [
        ('i', 0.35, 0.39),
        ('u', 0.4, 0.43),
        ('utorak', 0.43, 0.84),
        ('će', 0.84, 0.93),
        ('mjestimice', 0.93, 1.39),
        ('biti', 1.44, 1.67),
        ('kiše', 1.67, 2.08),
        ('češće', 2.24, 2.64),
        ('u', 2.64, 2.7),
        ('gorju', 2.74, 3.1),
        ('a', 3.15, 3.24),
        ('s', 3.24, 3.36),
        ('obzirom', 3.36, 3.75),
        ('na', 3.75, 3.82),
        ('to', 3.91, 4.07),
        ('da', 4.09, 4.2),
        ('će', 4.22, 4.35),
        ('do', 4.4, 4.47),
        ('nas', 4.47, 4.71),
        ('stići', 4.72, 4.96),
        ('hladniji', 4.98, 5.44),
        ('zrak', 5.45, 5.75),
    ]
}

primjer_6 = {
    'direktorij': 'primjer_6',
    'izvorni_tekst': 'polusatnom obraćanju javnosti u kojem je predsjednika'
                     ' hadezea andreja plenkovića prozvao za djelovanje'
                     ' suprotno stranačkim interesima',
    'intervali_rijeci': [
        ('polusatnom', 3.15, 3.63),
        ('obraćanju', 3.64, 4.13),
        ('javnosti', 4.18, 4.72),
        ('u', 4.73, 4.8),
        ('kojem', 4.86, 5.13),
        ('je', 5.13, 5.27),
        ('predsjednika', 5.35, 5.83),
        ('hadezea', 5.89, 6.41),
        ('andreja', 6.42, 6.78),
        ('plenkovića', 6.85, 7.4),
        ('prozvao', 7.83, 8.28),
        ('za', 8.28, 8.41),
        ('djelovanje', 8.46, 9.01),
        ('suprotno', 9.08, 9.55),
        ('stranačkim', 9.56, 10.12),
        ('interesima', 10.13, 10.79),
    ]
}


def prikazi_rezultate(
        podaci,
        interval=None,
        prikazi_tabelu_glasova=True,
        prikazi_prozodiju=True):
    if interval is None:
        interval = (podaci['intervali_rijeci'][0][1],
                    podaci['intervali_rijeci'][-1][2])

    analiza(podaci['izvorni_tekst'],
            podaci['intervali_rijeci'],
            interval,
            podaci['direktorij'],
            prikazi_tabelu_glasova,
            prikazi_prozodiju,
            margina=0)


# =============================================================================


# primjeri = [
#     primjer_1,
#     primjer_2,
#     primjer_3,
#     primjer_4,
#     primjer_5,
#     primjer_6,
# ]

# for d in primjeri:
#     poravnanje_titla(d)


# prikazi_rezultate(primjer_1, prikazi_tabelu_glasova=True,
#                   prikazi_prozodiju=False)
prikazi_rezultate(primjer_5, prikazi_tabelu_glasova=True,
                  prikazi_prozodiju=False)
# prikazi_rezultate(primjer_2, prikazi_tabelu_glasova=True,
#                   prikazi_prozodiju=False)
# prikazi_rezultate(primjer_3, prikazi_tabelu_glasova=True,
#                   prikazi_prozodiju=False)
# prikazi_rezultate(primjer_4, prikazi_tabelu_glasova=True,
#                   prikazi_prozodiju=False)
# prikazi_rezultate(primjer_6, prikazi_tabelu_glasova=False,
#                   prikazi_prozodiju=False)
